package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import org.apache.pdfbox.Loader;
import org.apache.pdfbox.io.RandomAccessReadBufferedFile;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

public class PdfUtil {
    /**
     * 将 PDF 的首页和尾页转换为 PNG 图片
     *
     * @param pdfFilepath 输入的 PDF 文件
     * @param outputDir   输出目录
     * @param dpi         图片分辨率（默认建议 300）
     * @return 生成的图片文件列表
     */
    public static List<String> convertFirstAndLastPagesToPng(String pdfFilepath, File outputDir, int dpi) throws IOException {
        List<String> generatedImages = ListUtil.list(false);

        try (PDDocument document = Loader.loadPDF(new RandomAccessReadBufferedFile(pdfFilepath))) {
            int totalPages = document.getNumberOfPages();
            if (totalPages == 0) {
                throw new IOException("PDF 文件无有效页面");
            }

            // 准备输出目录
            if (!outputDir.exists() && !outputDir.mkdirs()) {
                throw new IOException("无法创建输出目录: " + outputDir);
            }

            String baseName = FileUtil.mainName(pdfFilepath);
            PDFRenderer renderer = new PDFRenderer(document);

            // 始终处理首页（页码从1开始）
            generatedImages.add(
                    renderPage(renderer, 0, baseName + ".png", outputDir, dpi)
            );

            // 处理尾页（当总页数 > 1 时）
            if (totalPages > 1) {
                generatedImages.add(
                        renderPage(renderer, totalPages - 1,
                                baseName + "_footer.png",
                                outputDir, dpi)
                );
            }
        }

        return generatedImages;
    }

    /**
     * 渲染单页并保存为图片
     */
    private static String renderPage(PDFRenderer renderer, int pageIndex,
                                     String fileName, File outputDir, int dpi) throws IOException {
        BufferedImage image = renderer.renderImageWithDPI(pageIndex, dpi);
        File outputFile = new File(outputDir, fileName);
        ImageIO.write(image, "PNG", outputFile);
        return outputFile.getAbsolutePath();
    }
}
